@@ -15,7 +15,6 @@ Limitations:
   won't even have these special _NN variables set at all.
 """
 
-
 #-----------------------------------------------------------------------------
 # Module imports
 
@@ -50,119 +49,14 @@ from nose.util import anyp, getpackage, test_address, resolve_name, tolist
 
 #-----------------------------------------------------------------------------
 # Module globals and other constants
+#-----------------------------------------------------------------------------
 
 log = logging.getLogger(__name__)
 
-###########################################################################
-# *** HACK ***
-# We must start our own ipython object and heavily muck with it so that all the
-# modifications IPython makes to system behavior don't send the doctest
-# machinery into a fit.  This code should be considered a gross hack, but it
-# gets the job done.
-
-
-# Hack to modify the %run command so we can sync the user's namespace with the
-# test globals.  Once we move over to a clean magic system, this will be done
-# with much less ugliness.
-
-def _run_ns_sync(self,arg_s,runner=None):
-    """Modified version of %run that syncs testing namespaces.
-
-    This is strictly needed for running doctests that call %run.
-    """
-
-    out = _ip.IP.magic_run_ori(arg_s,runner)
-    _run_ns_sync.test_globs.update(_ip.user_ns)
-    return out
-
-
-class ipnsdict(dict):
-    """A special subclass of dict for use as an IPython namespace in doctests.
-
-    This subclass adds a simple checkpointing capability so that when testing
-    machinery clears it (we use it as the test execution context), it doesn't
-    get completely destroyed.
-    """
-    
-    def __init__(self,*a):
-        dict.__init__(self,*a)
-        self._savedict = {}
-        
-    def clear(self):
-        dict.clear(self)
-        self.update(self._savedict)
-
-    def _checkpoint(self):
-        self._savedict.clear()
-        self._savedict.update(self)
-
-    def update(self,other):
-        self._checkpoint()
-        dict.update(self,other)
-        # If '_' is in the namespace, python won't set it when executing code,
-        # and we have examples that test it.  So we ensure that the namespace
-        # is always 'clean' of it before it's used for test code execution.
-        self.pop('_',None)
-        
-
-def start_ipython():
-    """Start a global IPython shell, which we need for IPython-specific syntax.
-    """
-    import new
-
-    import IPython
-
-    def xsys(cmd):
-        """Execute a command and print its output.
-
-        This is just a convenience function to replace the IPython system call
-        with one that is more doctest-friendly.
-        """
-        cmd = _ip.IP.var_expand(cmd,depth=1)
-        sys.stdout.write(commands.getoutput(cmd))
-        sys.stdout.flush()
-
-    # Store certain global objects that IPython modifies
-    _displayhook = sys.displayhook
-    _excepthook = sys.excepthook
-    _main = sys.modules.get('__main__')
-
-    # Start IPython instance.  We customize it to start with minimal frills.
-    user_ns,global_ns = IPython.ipapi.make_user_namespaces(ipnsdict(),dict())
-    
-    IPython.Shell.IPShell(['--classic','--noterm_title'],
-                          user_ns,global_ns)
-
-    # Deactivate the various python system hooks added by ipython for
-    # interactive convenience so we don't confuse the doctest system
-    sys.modules['__main__'] = _main
-    sys.displayhook = _displayhook
-    sys.excepthook = _excepthook
-
-    # So that ipython magics and aliases can be doctested (they work by making
-    # a call into a global _ip object)
-    _ip = IPython.ipapi.get()
-    __builtin__._ip = _ip
-
-    # Modify the IPython system call with one that uses getoutput, so that we
-    # can capture subcommands and print them to Python's stdout, otherwise the
-    # doctest machinery would miss them.
-    _ip.system = xsys
-
-    # Also patch our %run function in.
-    im = new.instancemethod(_run_ns_sync,_ip.IP, _ip.IP.__class__)
-    _ip.IP.magic_run_ori = _ip.IP.magic_run
-    _ip.IP.magic_run = im
-
-# The start call MUST be made here.  I'm not sure yet why it doesn't work if
-# it is made later, at plugin initialization time, but in all my tests, that's
-# the case.
-start_ipython()
-
-# *** END HACK ***
-###########################################################################
 
+#-----------------------------------------------------------------------------
 # Classes and functions
+#-----------------------------------------------------------------------------
 
 def is_extension_module(filename):
     """Return whether the given filename is an extension module.
@@ -172,13 +66,19 @@ def is_extension_module(filename):
     return os.path.splitext(filename)[1].lower() in ('.so','.pyd')
 
 
-class nodoc(object):
+class DocTestSkip(object):
+    """Object wrapper for doctests to be skipped."""
+    
+    ds_skip = """Doctest to skip.
+    >>> 1 #doctest: +SKIP
+    """
+
     def __init__(self,obj):
         self.obj = obj
 
     def __getattribute__(self,key):
         if key == '__doc__':
-            return None
+            return DocTestSkip.ds_skip
         else:
             return getattr(object.__getattribute__(self,'obj'),key)
 
@@ -219,10 +119,10 @@ class DocTestFinder(doctest.DocTestFinder):
         Find tests for the given object and any contained objects, and
         add them to `tests`.
         """
-
+        #print '_find for:', obj, name, module  # dbg
         if hasattr(obj,"skip_doctest"):
             #print 'SKIPPING DOCTEST FOR:',obj  # dbg
-            obj = nodoc(obj)
+            obj = DocTestSkip(obj)
 
         doctest.DocTestFinder._find(self,tests, obj, name, module,
                                     source_lines, globs, seen)
@@ -318,6 +218,7 @@ class DocTestCase(doctests.DocTestCase):
         self._dt_optionflags = optionflags
         self._dt_checker = checker
         self._dt_test = test
+        self._dt_test_globs_ori = test.globs
         self._dt_setUp = setUp
         self._dt_tearDown = tearDown
 
@@ -327,8 +228,9 @@ class DocTestCase(doctests.DocTestCase):
         self._dt_runner = runner
 
 
-        # Each doctest should remember what directory it was loaded from...
-        self._ori_dir = os.getcwd()
+        # Each doctest should remember the directory it was loaded from, so
+        # things like %run work without too many contortions
+        self._ori_dir = os.path.dirname(test.filename)
 
     # Modified runTest from the default stdlib
     def runTest(self):
@@ -348,7 +250,8 @@ class DocTestCase(doctests.DocTestCase):
             # Save our current directory and switch out to the one where the
             # test was originally created, in case another doctest did a
             # directory change.  We'll restore this in the finally clause.
-            curdir = os.getcwd()
+            curdir = os.getcwdu()
+            #print 'runTest in dir:', self._ori_dir  # dbg
             os.chdir(self._ori_dir)
 
             runner.DIVIDER = "-"*70
@@ -363,16 +266,50 @@ class DocTestCase(doctests.DocTestCase):
 
     def setUp(self):
         """Modified test setup that syncs with ipython namespace"""
-        
+        #print "setUp test", self._dt_test.examples # dbg
         if isinstance(self._dt_test.examples[0],IPExample):
             # for IPython examples *only*, we swap the globals with the ipython
             # namespace, after updating it with the globals (which doctest
             # fills with the necessary info from the module being tested).
-            _ip.IP.user_ns.update(self._dt_test.globs)
-            self._dt_test.globs = _ip.IP.user_ns
+            _ip.user_ns.update(self._dt_test.globs)
+            self._dt_test.globs = _ip.user_ns
+            # IPython must protect the _ key in the namespace (it can't exist)
+            # so that Python's doctest code sets it naturally, so we enable
+            # this feature of our testing namespace.
+            _ip.user_ns.protect_underscore = True
 
-        doctests.DocTestCase.setUp(self)
-    
+        super(DocTestCase, self).setUp()
+
+    def tearDown(self):
+
+        # Undo the test.globs reassignment we made, so that the parent class
+        # teardown doesn't destroy the ipython namespace
+        if isinstance(self._dt_test.examples[0],IPExample):
+            self._dt_test.globs = self._dt_test_globs_ori
+            # Restore the behavior of the '_' key in the user namespace to
+            # normal after each doctest, so that unittests behave normally
+            _ip.user_ns.protect_underscore = False
+            
+        # XXX - fperez: I am not sure if this is truly a bug in nose 0.11, but
+        # it does look like one to me: its tearDown method tries to run
+        #
+        # delattr(__builtin__, self._result_var)
+        #
+        # without checking that the attribute really is there; it implicitly
+        # assumes it should have been set via displayhook.  But if the
+        # displayhook was never called, this doesn't necessarily happen.  I
+        # haven't been able to find a little self-contained example outside of
+        # ipython that would show the problem so I can report it to the nose
+        # team, but it does happen a lot in our code.
+        #
+        # So here, we just protect as narrowly as possible by trapping an
+        # attribute error whose message would be the name of self._result_var,
+        # and letting any other error propagate.
+        try:
+            super(DocTestCase, self).tearDown()
+        except AttributeError, exc:
+            if exc.args[0] != self._result_var:
+                raise
 
 
 # A simple subclassing of the original with a different class name, so we can
@@ -444,8 +381,12 @@ class IPDocTestParser(doctest.DocTestParser):
         """Convert input IPython source into valid Python."""
         out = []
         newline = out.append
-        for lnum,line in enumerate(source.splitlines()):
-            newline(_ip.IP.prefilter(line,lnum>0))
+        #print 'IPSRC:\n',source,'\n###'  # dbg
+        # The input source must be first stripped of all bracketing whitespace
+        # and turned into lines, so it looks to the parser like regular user
+        # input
+        for lnum,line in enumerate(source.strip().splitlines()):
+            newline(_ip.prefilter(line,lnum>0))
         newline('')  # ensure a closing newline, needed by doctest
         #print "PYSRC:", '\n'.join(out)  # dbg
         return '\n'.join(out)
@@ -637,8 +578,11 @@ class IPDocTestRunner(doctest.DocTestRunner,object):
         # attribute.  Our new %run will then only make the namespace update
         # when called (rather than unconconditionally updating test.globs here
         # for all examples, most of which won't be calling %run anyway).
-        _run_ns_sync.test_globs = test.globs
+        #_ip._ipdoctest_test_globs = test.globs
+        #_ip._ipdoctest_test_filename = test.filename
 
+        test.globs.update(_ip.user_ns)
+        
         return super(IPDocTestRunner,self).run(test,
                                                compileflags,out,clear_globs)
 
@@ -656,6 +600,23 @@ class ExtensionDoctest(doctests.Doctest):
     name = 'extdoctest'   # call nosetests with --with-extdoctest
     enabled = True
 
+    def __init__(self,exclude_patterns=None):
+        """Create a new ExtensionDoctest plugin.
+
+        Parameters
+        ----------
+
+        exclude_patterns : sequence of strings, optional
+          These patterns are compiled as regular expressions, subsequently used
+          to exclude any filename which matches them from inclusion in the test
+          suite (using pattern.search(), NOT pattern.match() ).
+        """
+
+        if exclude_patterns is None:
+            exclude_patterns = []
+        self.exclude_patterns = map(re.compile,exclude_patterns)
+        doctests.Doctest.__init__(self)
+
     def options(self, parser, env=os.environ):
         Plugin.options(self, parser, env)
         parser.add_option('--doctest-tests', action='store_true',
@@ -688,6 +649,7 @@ class ExtensionDoctest(doctests.Doctest):
         self.globs = None
         self.extraglobs = None
 
+
     def loadTestsFromExtensionModule(self,filename):
         bpath,mod = os.path.split(filename)
         modname = os.path.splitext(mod)[0]
@@ -703,8 +665,8 @@ class ExtensionDoctest(doctests.Doctest):
     # a  few modifications to control output checking.
 
     def loadTestsFromModule(self, module):
-        #print 'lTM',module  # dbg
-
+        #print '*** ipdoctest - lTM',module  # dbg
+        
         if not self.matches(module.__name__):
             log.debug("Doctest doesn't want module %s", module)
             return
@@ -733,8 +695,7 @@ class ExtensionDoctest(doctests.Doctest):
 
 
     def loadTestsFromFile(self, filename):
-        #print 'lTF',filename  # dbg
-
+        #print "ipdoctest - from file", filename # dbg
         if is_extension_module(filename):
             for t in self.loadTestsFromExtensionModule(filename):
                 yield t
@@ -761,23 +722,11 @@ class ExtensionDoctest(doctests.Doctest):
         Modified version that accepts extension modules as valid containers for
         doctests.
         """
-        print 'Filename:',filename  # dbg
+        #print '*** ipdoctest- wantFile:',filename  # dbg
 
-        # XXX - temporarily hardcoded list, will move to driver later
-        exclude = ['IPython/external/',
-                   'IPython/platutils_win32',
-                   'IPython/frontend/cocoa',
-                   'IPython_doctest_plugin',
-                   'IPython/Gnuplot',
-                   'IPython/Extensions/ipy_',
-                   'IPython/Extensions/PhysicalQIn',
-                   'IPython/Extensions/scitedirector',
-                   'IPython/testing/plugin',
-                   ]
-
-        for fex in exclude:
-            if fex in filename:  # substring
-                #print '###>>> SKIP:',filename  # dbg
+        for pat in self.exclude_patterns:
+            if pat.search(filename):
+                # print '###>>> SKIP:',filename  # dbg
                 return False
 
         if is_extension_module(filename):
@@ -792,15 +741,53 @@ class IPythonDoctest(ExtensionDoctest):
     name = 'ipdoctest'   # call nosetests with --with-ipdoctest
     enabled = True
 
-    def configure(self, options, config):
+    def makeTest(self, obj, parent):
+        """Look for doctests in the given object, which will be a
+        function, method or class.
+        """
+        #print 'Plugin analyzing:', obj, parent  # dbg
+        # always use whitespace and ellipsis options
+        optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
 
+        doctests = self.finder.find(obj, module=getmodule(parent))
+        if doctests:
+            for test in doctests:
+                if len(test.examples) == 0:
+                    continue
+
+                yield DocTestCase(test, obj=obj,
+                                  optionflags=optionflags,
+                                  checker=self.checker)
+
+    def options(self, parser, env=os.environ):
+        #print "Options for nose plugin:", self.name # dbg
+        Plugin.options(self, parser, env)
+        parser.add_option('--ipdoctest-tests', action='store_true',
+                          dest='ipdoctest_tests',
+                          default=env.get('NOSE_IPDOCTEST_TESTS',True),
+                          help="Also look for doctests in test modules. "
+                          "Note that classes, methods and functions should "
+                          "have either doctests or non-doctest tests, "
+                          "not both. [NOSE_IPDOCTEST_TESTS]")
+        parser.add_option('--ipdoctest-extension', action="append",
+                          dest="ipdoctest_extension",
+                          help="Also look for doctests in files with "
+                          "this extension [NOSE_IPDOCTEST_EXTENSION]")
+        # Set the default as a list, if given in env; otherwise
+        # an additional value set on the command line will cause
+        # an error.
+        env_setting = env.get('NOSE_IPDOCTEST_EXTENSION')
+        if env_setting is not None:
+            parser.set_defaults(ipdoctest_extension=tolist(env_setting))
+
+    def configure(self, options, config):
+        #print "Configuring nose plugin:", self.name # dbg
         Plugin.configure(self, options, config)
-        self.doctest_tests = options.doctest_tests
-        self.extension = tolist(options.doctestExtension)
+        self.doctest_tests = options.ipdoctest_tests
+        self.extension = tolist(options.ipdoctest_extension)
 
         self.parser = IPDocTestParser()
         self.finder = DocTestFinder(parser=self.parser)
         self.checker = IPDoctestOutputChecker()
         self.globs = None
         self.extraglobs = None
-
