@@ -5,7 +5,6 @@
 import inspect
 
 # Our own
-from IPython.testing import decorators as dec
 
 #-----------------------------------------------------------------------------
 # Testing functions
@@ -38,14 +37,10 @@ def doctest_ivars():
     In [6]: zz
     Out[6]: 1
     """
-    
-@dec.skip_doctest
+
 def doctest_refs():
     """DocTest reference holding issues when running scripts.
 
     In [32]: run show_refs.py
     c referrers: [<type 'dict'>]
-
-    In [33]: map(type,gc.get_referrers(c))
-    Out[33]: [<type 'dict'>]
     """
